﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using RedisAndEntityFrameworkInWebApi.Data;
using StackExchange.Redis;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;

namespace RedisAndEntityFrameworkInWebApi.Controllers
{
    [Route("[controller]")]
    [ApiController]
    public class ValuesController : ControllerBase
    {
        private readonly IConnectionMultiplexer _connectionMultiplexer;
        private readonly KeyAndValueContext _keyAndValueContext;


        public ValuesController(IConnectionMultiplexer connectionMultiplexer, KeyAndValueContext keyAndValueContext)
        {
            _connectionMultiplexer = connectionMultiplexer;
            _keyAndValueContext = keyAndValueContext;
        }

        [HttpGet("{key}")]
        public async Task<IActionResult> Get(string key)
        {
            var tokenSource = new CancellationTokenSource();
            var cancellationToken = tokenSource.Token;

            var redis = _connectionMultiplexer.GetDatabase();
            Task<KeyAndValue> dbTask = _keyAndValueContext.KeyAndValues.Where(kv => kv.Key == key).FirstAsync(cancellationToken);
            Task<RedisValue> redisTask = redis.StringGetAsync(key);

            string value = "";
            var completedTask = await Task.WhenAny(dbTask, redisTask);

            if (completedTask is Task<RedisValue> redisTaskCompleted)
            {
                value = await redisTaskCompleted;
                if (value != null)
                {
                    value = $"key:{key}, value:{value}, source:Redis";
                    tokenSource.Cancel();
                }
                else
                {
                    completedTask = await Task.WhenAny(dbTask);
                }
            }

            if (completedTask is Task<KeyAndValue> dbTaskCompleted)
            {
                value = $"key:{key}, value:{(await dbTaskCompleted).Value}, source:MSSQL";
            }

            return Ok(value);
        }
    }

}
